//-----------------------------------------------------------------------------
//  HAMEG Arbitrary Function Generator
//  VXIpnp Instrument Driver
//  Original Release: September 2010
//  By: Nabil Ouddane
//
//  Should you have any technical questions please contact the hotline of
//  HAMEG Support Center
//
//  e-mail: support@hameg.com
//
//  Version: 1.0.0
//
//  Modification History:
//     see ChangeLog
 //----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Headers
//-----------------------------------------------------------------------------
#include <math.h>
#include <float.h>
#include "hmf2500.h"
#include "hmf2500_utility.h"

//-----------------------------------------------------------------------------
// Constants
//-----------------------------------------------------------------------------
static ViString HighLowArr[] = {"LOW","HIGH",VI_NULL};
static ViString ModulationArr[] = {"AM","FM","PM","PWM",VI_NULL};
static ViString SweepArr[] = {"Center","Span","Start","Stop",VI_NULL};
static ViString ArgumentArr[] = {"ARB1","ARB2","ARB3","ARB4",VI_NULL};
static ViString EdgeArr[] = {"RISE","FALL",VI_NULL};
//-----------------------------------------------------------------------------
// Callback Declarations
//-----------------------------------------------------------------------------
ViStatus hmf2500_CheckStatusCallback (ViSession io);
ViStatus hmf2500_WaitForOPCCallback (ViSession io);
ViStatus _VI_FUNC  hmf2500_RsClose (ViSession vi);

//-----------------------------------------------------------------------------
// Initialize                                                
//-----------------------------------------------------------------------------
/// HIFN This function performs the following initialization actions:
/// HIPAR resourceName/Pass the resource name of the device to initialize. 
/// HIPAR IDQuery/Creates a new instrument driver session.
/// HIPAR resetDevice/Specify whether you want the instrument driver to perform an IDQuery.
/// HIPAR newInstrSession/Returns a ViSession handle that you use to identify the
/// HIPAR newInstrSession/instrument in all subsequent instrument driver function calls.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_init (ViRsrc resourceName, ViBoolean IDQuery,
                                 ViBoolean resetDevice, ViSession *newInstrSession)
{
    ViStatus    error = VI_SUCCESS;

    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                          VI_ERROR_PARAMETER4, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    checkErr( hmf2500_InitWithOptions (resourceName, IDQuery, resetDevice,
                                      "", newInstrSession));

Error:
    return error;
}

//-----------------------------------------------------------------------------
// Initialize With Options                                                
//-----------------------------------------------------------------------------
/// HIFN This function performs the following initialization actions:
/// HIPAR resourceName/Pass the resource name of the device to initialize. 
/// HIPAR IDQuery/Creates a new instrument driver session.
/// HIPAR resetDevice/Specify whether you want the instrument driver to perform an IDQuery.
/// HIPAR newInstrSession/Returns a ViSession handle that you use to identify the
/// HIPAR newInstrSession/instrument in all subsequent instrument driver function calls.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_InitWithOptions(
    ViRsrc          resourceName,
    ViBoolean       IDQuery,
    ViBoolean       resetDevice,
    ViString        optionString,
    ViSession      *newInstrSession
)
{
    ViStatus                error = VI_SUCCESS;
    ViChar                  newResourceName[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar                  newOptionString[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViBoolean               isLogicalName = 0;
    ViSession               instrSession = 0, rmSession = 0;
    RsAttrPropertiesPtr     *attrList=NULL;
    RsRepCapPtr             repCapTable = NULL;
    ViUInt16                interface = VI_INTF_GPIB;
	IDQuery; // not used

    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                         VI_ERROR_PARAMETER5, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    /* Prepare resource manager session handle */
    checkErr (viOpenDefaultRM (&rmSession));

    /* Establish communication session with the instrument */

    viCheckErr( viOpen (rmSession, resourceName, VI_LOAD_CONFIG, VI_NULL, &instrSession));
    ///viCheckErr( viPrintf (instrSession, "*RST\n"));
    viCheckErr( viFlush(instrSession,VI_READ_BUF));
        
    
    viCheckErr( viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &interface));
    
    /* Configure VISA Formatted I/O */
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 15000 ));
    viCheckErr( viSetBuf (instrSession, (ViUInt16) (VI_READ_BUF | VI_WRITE_BUF), 4000));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_WR_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_RD_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));

    if (interface == VI_INTF_GPIB) {
        viCheckErr( viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE));
        viCheckErr( viSetAttribute (instrSession, VI_ATTR_SEND_END_EN, VI_TRUE));
    }

#ifndef VI_INTF_TCPIP
    if (interface == 6) {
#else
    if (interface == VI_INTF_TCPIP) {
#endif
        //viCheckErr( viSetAttribute (instrSession, VI_ATTR_TERMCHAR, '\n'));
        viCheckErr( viSetAttribute (instrSession, VI_ATTR_TERMCHAR, 0x0a));
        //viCheckErr( viSetAttribute (instrSession, VI_ATTR_SUPPRESS_END_EN, VI_TRUE));
        viCheckErr( viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE));
    }
    
    
    
    checkErr( Rs_GetInfoFromResourceName (resourceName,
                                          (ViString)optionString,
                                          newResourceName,
                                          newOptionString,
                                          &isLogicalName));

    ATTRLIST(attrList,hmf2500); 
    checkErr( Rs_SpecificDriverNew (instrSession, "hmf2500", newOptionString, attrList));
   
    /* Build repeated capability table */
    REPCAPTABLE(repCapTable,hmf2500);
    checkErr( Rs_BuildRepCapTable (instrSession, repCapTable));

    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_VISA_RM_SESSION, 0, &rmSession));

    /* With remote control via Ethernet, remote control is not automatically
       set by means of a command. The instrument must be explicitly set to
       the REMOTE state, e.g. by sending the intf command &GTR
    */
    //checkErr (viWrite (instrSession, (ViBuf) "&GTR\n", 5, NULL));
    
    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice == VI_TRUE)
    {
        checkErr(hmf2500_reset (instrSession));
    }
    else  /*- Send Default Instrument Setup ---------------------------------*/
        checkErr( hmf2500_DefaultInstrSetup (instrSession));


    /* Instrument session accessible by the end user is native VISA session,
       or when simulated virtual instrument (vi) session. */
    checkErr( hmf2500_CheckStatus (instrSession));
    

Error:

    if (error < VI_SUCCESS)
    {
        ViSession   rmSess = VI_NULL;

        (void) Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSess), &rmSess);
        
        if (instrSession != 0)
            (void) viClose (instrSession);
        (void) viClose (rmSess);
    }
    else
    {
        *newInstrSession = instrSession;
    }

    if (error < VI_SUCCESS)
        (void) Rs_Dispose (*newInstrSession);

    return error;
}

//----------------------------------------------------------------------------
// Configure RF
//----------------------------------------------------------------------------
/// HIFN  Sets the parameters for RF.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR frequency/Sets the output frequency.
/// HIPAR powerLevel/Sets the output amplitude.
/// HIPAR amplitudeUnit/Changes the unit of the amplitude from Volt to dBm or from dBm to Volt.
/// HIPAR input/Reserved for future use.	
//----------------------------------------------------------------------------  
ViStatus _VI_FUNC hmf2500_ConfigureRF (ViSession instrSession,
                                       ViReal64 frequency, ViReal64 powerLevel,
                                       ViInt32 amplitudeUnit,
									   ViInt32 input)
{
    ViStatus    error = VI_SUCCESS;
	input;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(hmf2500_SetAttributeViReal64(instrSession, "",
        HMF2500_ATTR_FREQUENCY, frequency), 2, "Frequency");
    
    viCheckParm(hmf2500_SetAttributeViReal64(instrSession, "",
        HMF2500_ATTR_POWER_LEVEL, powerLevel), 3, "Power Level");
    
    viCheckParm(hmf2500_SetAttributeViInt32(instrSession, "",
        HMF2500_ATTR_AMPLITUDE_UNIT, amplitudeUnit), 4, "Amplitude Unit");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

//-----------------------------------------------------------------------------
// Configure Period
//-----------------------------------------------------------------------------
/// HIFN  Sets the output period.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR period/Sets the output period.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ConfigurePeriod (ViSession instrSession,
                                           ViReal64 period)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(hmf2500_SetAttributeViReal64(instrSession, "",
        HMF2500_ATTR_PERIOD, period), 2, "Period");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

//-----------------------------------------------------------------------------
// Configure Output Enabled
//-----------------------------------------------------------------------------
/// HIFN This function activates or deactivates the output.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR outputEnabled/Activates or deactivates the output.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ConfigureOutputEnabled (ViSession instrSession,
                                                  ViBoolean outputEnabled)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL)); 

    viCheckParm(hmf2500_SetAttributeViBoolean(instrSession, "",
        HMF2500_ATTR_OUTPUT_ENABLED, outputEnabled), 2, "Output Enabled");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

//-----------------------------------------------------------------------------
// Configure Power Level Offset 
//-----------------------------------------------------------------------------
/// HIFN  Sets the power Level Offset. 
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function.
/// HIPAR powerLevel/Activates or deactivates the offset voltage.
/// HIPAR powerLevelOffset/Sets the output offset.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ConfigurePowerLevelOffset (ViSession instrSession,
                                                     ViBoolean powerLevel,
                                                     ViReal64 powerLevelOffset)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));    

    viCheckParm(hmf2500_SetAttributeViBoolean(instrSession, "",
        HMF2500_ATTR_POWER_LEVEL_OFFSET_ENABLED, powerLevel), 2, "Power Level");
    
    viCheckParm(hmf2500_SetAttributeViReal64(instrSession, "",
        HMF2500_ATTR_POWER_LEVEL_OFFSET, powerLevelOffset), 3, "Power Level Offset");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);   
    return error;
}  

//-----------------------------------------------------------------------------
// Configure Power Level With Offset
//-----------------------------------------------------------------------------
/// HIFN  Sets the upper and the lower level of the function amplitude.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR highLow/Selects the appropriate repeated capability.
/// HIPAR powerLevelOffset/Sets the upper and the lower level of the function amplitude.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ConfigurePowerLevelWithOffset (ViSession instrSession,
                                                         ViInt32 highLow,
                                                         ViReal64 powerLevelOffset) 
                                                             
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    if (hmf2500_invalidViInt32Range (highLow, 0, 1) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "HighLow");

    viCheckParm(hmf2500_SetAttributeViReal64(instrSession, HighLowArr[highLow],
        HMF2500_ATTR_POWER_LEVEL_WITH_OFFSET, powerLevelOffset), 3, "Power Level Offset");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

//-----------------------------------------------------------------------------
// Configure System Reference
//-----------------------------------------------------------------------------
/// HIFN  Sets the 10MHz system reference clock to internal (immediate) or external.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR systemReference/sets the 10MHz system reference clock to internal (immediate) or external.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ConfigureSystemReference (ViSession instrSession,
                                                    ViInt32 systemReference)
{
	ViStatus	error = VI_SUCCESS;
	checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(hmf2500_SetAttributeViInt32(instrSession, "",
        HMF2500_ATTR_SYSTEM_REFERENCE, systemReference), 2, "System Reference");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
	return error;
}

//-----------------------------------------------------------------------------
// Configure Output Function 
//-----------------------------------------------------------------------------
/// HIFN  Selects the output function.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR outputFunction/Selects the output function.
//-----------------------------------------------------------------------------   
ViStatus _VI_FUNC hmf2500_ConfigureOutputFunction (ViSession instrSession,
                                                   ViInt32 outputFunction)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(hmf2500_SetAttributeViInt32(instrSession, "",
        HMF2500_ATTR_OUTPUT_FUNCTION, outputFunction), 2, "Output Function");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);  
    return error;
}

//-----------------------------------------------------------------------------
// Configure Output
//-----------------------------------------------------------------------------
/// HIFN  Sets the output parameters.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR outputLoad/Selects the output load.
/// HIPAR polarity/Inverts the current output waveform (pulse or arbitrary only).
//-----------------------------------------------------------------------------   
ViStatus _VI_FUNC hmf2500_ConfigureOutput (ViSession instrSession,
                                           ViInt32 outputLoad, ViInt32 polarity)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(hmf2500_SetAttributeViInt32(instrSession, "",
        HMF2500_ATTR_OUTPUT_LOAD, outputLoad), 2, "Output Load");
    
    viCheckParm(hmf2500_SetAttributeViInt32(instrSession, "",
        HMF2500_ATTR_OUTPUT_POLARITY, polarity), 3, "Polarity");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

//-----------------------------------------------------------------------------
// Configure Square Duty Cycle  
//-----------------------------------------------------------------------------
/// HIFN  Sets the output duty cycle.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR squareDutyCycle/Sets the output duty cycle.
//-----------------------------------------------------------------------------  
ViStatus _VI_FUNC hmf2500_ConfigureSquareDutyCycle (ViSession instrSession,
                                                    ViReal64 squareDutyCycle)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL)); 

    viCheckParm(hmf2500_SetAttributeViReal64(instrSession, "",
        HMF2500_ATTR_SQUARE_DUTY_CYCLE, squareDutyCycle), 2, "Square Duty Cycle");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

//-----------------------------------------------------------------------------
// Configure Square Width
//-----------------------------------------------------------------------------
/// HIFN  This function sets the low and the high width value for the square signal.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR squareSide/This control selects the square side to be configured.
/// HIPAR width/This control sets the square width value.
//-----------------------------------------------------------------------------  
ViStatus _VI_FUNC hmf2500_ConfigureSquareWidth (ViSession instrSession,
                                                ViInt32 squareSide, ViReal64 width)
{
	ViStatus	error = VI_SUCCESS;
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    if (hmf2500_invalidViInt32Range (squareSide, 0, 1) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "squareSide");
    
    viCheckParm(hmf2500_SetAttributeViReal64(instrSession, HighLowArr[squareSide],
        HMF2500_ATTR_SQUARE_WIDTH, width), 4, "Width");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
	return error;
}

//-----------------------------------------------------------------------------
// Configure Ramp Symmetry  
//-----------------------------------------------------------------------------
/// HIFN  Sets the output symmetry.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR rampSymmetry/Sets the output symmetry.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ConfigureRampSymmetry (ViSession instrSession,
                                                 ViReal64 rampSymmetry)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL)); 

    viCheckParm(hmf2500_SetAttributeViReal64(instrSession, "",
        HMF2500_ATTR_RAMP_SYMMETRY, rampSymmetry), 2, "Ramp Symmetry");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

//-----------------------------------------------------------------------------
// Configure Ramp Edge Time 
//-----------------------------------------------------------------------------
/// HIFN  Sets the ramp edge time.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR edge/Selects the ramp edge.
/// HIPAR time/Sets the edge time.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ConfigureRampEdgeTime (ViSession instrSession,
                                                 ViInt32 edge, ViReal64 time)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL)); 

    if (hmf2500_invalidViInt32Range (edge, 0, 1) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Edge");
	
    viCheckParm(hmf2500_SetAttributeViReal64(instrSession, EdgeArr[edge],
        HMF2500_ATTR_RAMP_EDGE_TIME, time), 2, "Time");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
	return error;
}

//-----------------------------------------------------------------------------
// Configure Pulse   
//-----------------------------------------------------------------------------
/// HIFN  Sets the pulse parameters.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR highLow/Selects the appropriate repeated capability.
/// HIPAR pulseDutyCycle/Sets the output duty cycle.
/// HIPAR pulseWidth/Sets the output width for the upper or lower part of the pulse function.
/// HIPAR pulseEdgeTime/Sets the output edge times.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ConfigurePulse (ViSession instrSession,
                                          ViInt32 highLow,
                                          ViReal64 pulseDutyCycle,
                                          ViReal64 pulseWidth,
                                          ViReal64 pulseEdgeTime
                                          )
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    if (hmf2500_invalidViInt32Range (highLow, 0, 1) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "HighLow");

    viCheckParm(hmf2500_SetAttributeViReal64(instrSession, "",
        HMF2500_ATTR_PULSE_DUTY_CYCLE, pulseDutyCycle), 3, "Pulse Duty Cycle");
    
    viCheckParm(hmf2500_SetAttributeViReal64(instrSession, HighLowArr[highLow],
        HMF2500_ATTR_PULSE_WIDTH, pulseWidth), 4, "Pulse Width");
    
    viCheckParm(hmf2500_SetAttributeViReal64(instrSession, "",
        HMF2500_ATTR_PULSE_EDGE_TIME, pulseEdgeTime), 5, "Pulse Edge Time");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

//-----------------------------------------------------------------------------
// Configure Trigger   
//-----------------------------------------------------------------------------
/// HIFN  Configures the trigger.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR triggerEnabled/Acitvates or deactivates the trigger function.
/// HIPAR systemReference/Sets the trigger source.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ConfigureTrigger (ViSession instrSession,
                                            ViBoolean triggerEnabled,
                                            ViInt32 systemReference)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));  

    viCheckParm(hmf2500_SetAttributeViBoolean(instrSession, "",
        HMF2500_ATTR_TRIGGER_ENABLED, triggerEnabled), 2, "Trigger Enabled");
    
	if (triggerEnabled == VI_TRUE);
	{
    viCheckParm(hmf2500_SetAttributeViInt32(instrSession, "",
        HMF2500_ATTR_TRIGGER_SOURCE, systemReference), 3, "System Reference");
    }
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

//-----------------------------------------------------------------------------
// Configure Trigger   
//-----------------------------------------------------------------------------
/// HIFN  Configure Trigger Slope.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR triggerSlope/Sets the trigger source.
/// HIPAR outputTriggerSlope/Sets the output trigger source.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ConfigureTriggerSlope (ViSession instrSession,
                                                 ViInt32 triggerSlope,
                                                 ViInt32 outputTriggerSlope)
{
	ViStatus	error = VI_SUCCESS;
	checkErr (Rs_LockSession (instrSession, VI_NULL));  

    viCheckParm(hmf2500_SetAttributeViInt32(instrSession, "",
        HMF2500_ATTR_TRIGGER_SLOPE, triggerSlope), 2, "Trigger Slope");
    
    viCheckParm(hmf2500_SetAttributeViInt32(instrSession, "",
        HMF2500_ATTR_OUTPUT_TRIGGER_SLOPE, outputTriggerSlope), 3, "Output Trigger Slope");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 

	return error;
}

//-----------------------------------------------------------------------------
// Configure Modulation Shape    
//-----------------------------------------------------------------------------
/// HIFN  Sets the shape of the modulation signal.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR modulation/Selects the appropriate modulation for the repeated capability.
/// HIPAR modulationShape/Sets the shape of the modulation signal.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ConfigureModulationShape (ViSession instrSession,
                                                    ViInt32 modulation,
                                                    ViInt32 modulationShape
                                                    )
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
     if (hmf2500_invalidViInt32Range (modulation, 0, 3) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Modulation");
    
    viCheckParm(hmf2500_SetAttributeViInt32(instrSession, ModulationArr[modulation],
        HMF2500_ATTR_MODULATION_SHAPE, modulationShape), 3, "Modulation Shape");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

//-----------------------------------------------------------------------------
// Configure AM    
//-----------------------------------------------------------------------------
/// HIFN  Configures the AM parameters.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR AMEnabled/Activates or deactivates AM modulation of the output signal.
/// HIPAR AMSource/Sets the modulation source.
/// HIPAR AMDepth/Sets the AM modulation depth in %.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ConfigureAM (ViSession instrSession,
                                       ViBoolean AMEnabled, ViInt32 AMSource,
                                       ViReal64 AMDepth)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(hmf2500_SetAttributeViBoolean(instrSession, "",
        HMF2500_ATTR_AM_ENABLED, AMEnabled), 2, "AM Enabled");
    
    viCheckParm(hmf2500_SetAttributeViInt32(instrSession, "",
        HMF2500_ATTR_AM_SOURCE, AMSource), 3, "AM Source");
    
    viCheckParm(hmf2500_SetAttributeViReal64(instrSession, "",
        HMF2500_ATTR_AM_DEPTH, AMDepth), 4, "AM Depth");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);  
    return error;
}

//-----------------------------------------------------------------------------
// Configure FM    
//-----------------------------------------------------------------------------
/// HIFN  Configures the FM parameters.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR FMEnabled/Activates or deactivates FM modulation of the output signal.
/// HIPAR FMSource/Sets the modulation source.
/// HIPAR FMDeviation/Sets the peak frequency deviation in Hertz.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ConfigureFM (ViSession instrSession,
                                       ViBoolean FMEnabled, ViInt32 FMSource,
                                       ViReal64 FMDeviation)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(hmf2500_SetAttributeViBoolean(instrSession, "",
        HMF2500_ATTR_FM_ENABLED, FMEnabled), 2, "FM Enabled");
    
    viCheckParm(hmf2500_SetAttributeViInt32(instrSession, "",
        HMF2500_ATTR_FM_SOURCE, FMSource), 3, "FM Source");
    
    viCheckParm(hmf2500_SetAttributeViReal64(instrSession, "",
        HMF2500_ATTR_FM_DEVIATION, FMDeviation), 4, "FM Deviation");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

//-----------------------------------------------------------------------------
// Configure PM    
//-----------------------------------------------------------------------------
/// HIFN  Configures the PM parameters.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR PMEnabled/Activates or deactivates PM modulation of the output signal.
/// HIPAR PMSource/Sets the modulation source.
/// HIPAR PMDeviation/Sets the deviation of the phase in degree (-180 to+180).
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ConfigurePM (ViSession instrSession,
                                       ViBoolean PMEnabled, ViInt32 PMSource,
                                       ViReal64 PMDeviation)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(hmf2500_SetAttributeViBoolean(instrSession, "",
        HMF2500_ATTR_PM_ENABLED, PMEnabled), 2, "PM Enabled");
    
    viCheckParm(hmf2500_SetAttributeViInt32(instrSession, "",
        HMF2500_ATTR_PM_SOURCE, PMSource), 3, "PM Source");
    
    viCheckParm(hmf2500_SetAttributeViReal64(instrSession, "",
        HMF2500_ATTR_PM_DEVIATION, PMDeviation), 4, "PM Deviation");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
    
}

//-----------------------------------------------------------------------------
// Configure PWM    
//-----------------------------------------------------------------------------
/// HIFN  Configures the PWM parameters.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR pulseEnabled/Sets the PWM modulation status.
/// HIPAR pulseSource/Sets the modulation source.
/// HIPAR pulseModulationDutyCycle/Sets the pulse modulation duty cycle.
/// HIPAR internalFrequency/Sets the frequency of the PWM modulation signal.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ConfigurePWM (ViSession instrSession,
                                        ViBoolean pulseEnabled,
                                        ViInt32 pulseSource,
                                        ViReal64 pulseModulationDutyCycle,
                                        ViReal64 internalFrequency)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(hmf2500_SetAttributeViBoolean(instrSession, "",
        HMF2500_ATTR_PULSE_ENABLED, pulseEnabled), 2, "Pulse Enabled");
    
    viCheckParm(hmf2500_SetAttributeViInt32(instrSession, "",
        HMF2500_ATTR_PULSE_SOURCE, pulseSource), 3, "Pulse Source");
    
    viCheckParm(hmf2500_SetAttributeViReal64(instrSession, "",
        HMF2500_ATTR_PULSE_MODULATION_DUTY_CYCLE, pulseModulationDutyCycle), 4, "Pulse Modulation Duty Cycle");
    
    viCheckParm(hmf2500_SetAttributeViReal64(instrSession, "",
        HMF2500_ATTR_PULSE_INTERNAL_FREQUENCY, internalFrequency), 5, "Internal Frequency");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;

}

//-----------------------------------------------------------------------------
// Configure FSK Modulation    
//-----------------------------------------------------------------------------
/// HIFN  Configures the FSK modulation. 
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR FSKEnabled/Sets FSK modulation state.
/// HIPAR FSKSource/Sets the modulation source.
/// HIPAR FSKDutyCycle/Sets the FSK modulation duty cycle.
/// HIPAR FSKFrequency/Sets the frequency of the FSK modulation signal.
/// HIPAR FSKHop/sets the rate (frequency) of the FSK modulation in order to define the timeframe between the frequency shifts.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ConfigureFSKModulation (ViSession instrSession,
                                                  ViBoolean FSKEnabled,
                                                  ViInt32 FSKSource,
                                                  ViReal64 FSKDutyCycle,
                                                  ViReal64 FSKFrequency,
                                                  ViReal64 FSKHop)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(hmf2500_SetAttributeViBoolean(instrSession, "",
        HMF2500_ATTR_FSK_ENABLED, FSKEnabled), 2, "FSK Enabled");
    
    viCheckParm(hmf2500_SetAttributeViInt32(instrSession, "",
        HMF2500_ATTR_FSK_SOURCE, FSKSource), 3, "FSK Source");
    
    viCheckParm(hmf2500_SetAttributeViReal64(instrSession, "",
        HMF2500_ATTR_FSK_DUTY_CYCLE, FSKDutyCycle), 4, "FSK Duty Cycle");
    
    viCheckParm(hmf2500_SetAttributeViReal64(instrSession, "",
        HMF2500_ATTR_FSK_FREQUENCY, FSKFrequency), 5, "FSK Frequency");
    
    viCheckParm(hmf2500_SetAttributeViReal64(instrSession, "",
        HMF2500_ATTR_FSK_HOP, FSKHop), 6, "FSK Hop");

Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

//-----------------------------------------------------------------------------
// Configure Frequency Sweep    
//-----------------------------------------------------------------------------
/// HIFN  Configures the frequency sweep. 
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR sweep/selects the appropriate sweep for the repeated capability.
/// HIPAR sweepEnabled/Activates or deactivates sweep mode.
/// HIPAR sweepFrequency/Sets the frequency sweep range.
/// HIPAR sweepSpacing/Sets the spacing of the sweep to linear or logarithmic.
/// HIPAR sweepTime/Sets the time required to sweep from the start frequency to the stop frequency. 
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ConfigureFrequencySweep (ViSession instrSession,
                                                   ViInt32 sweep,
                                                   ViBoolean sweepEnabled,
                                                   ViReal64 sweepFrequency,
                                                   ViInt32 sweepSpacing,
                                                   ViReal64 sweepTime)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    if (hmf2500_invalidViInt32Range (sweep, 0, 3) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Sweep");
    
    viCheckParm(hmf2500_SetAttributeViBoolean(instrSession, "",
        HMF2500_ATTR_SWEEP_ENABLED, sweepEnabled), 3, "Sweep Enabled");
    
    viCheckParm(hmf2500_SetAttributeViReal64(instrSession, SweepArr[sweep],
        HMF2500_ATTR_SWEEP_FREQUENCY, sweepFrequency), 4, "Sweep Frequency");
    
    viCheckParm(hmf2500_SetAttributeViReal64(instrSession, "",
        HMF2500_ATTR_SWEEP_SPACING, sweepSpacing), 5, "Sweep Spacing");
    
    viCheckParm(hmf2500_SetAttributeViReal64(instrSession, "",
        HMF2500_ATTR_SWEEP_TIME, sweepTime), 6, "Sweep Time");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);     
    return error;
}

//-----------------------------------------------------------------------------
// Configure Sweep Marker   
//-----------------------------------------------------------------------------
/// HIFN  Configures the sweep marker. 
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR sweepMarkerEnabled/Disables or enables the frequency marker.
/// HIPAR sweepMarkerFrequency/Sets the frequency of the marker.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ConfigureSweepMarker (ViSession instrSession,
                                                ViBoolean sweepMarkerEnabled,
                                                ViReal64 sweepMarkerFrequency)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(hmf2500_SetAttributeViBoolean(instrSession, "",
        HMF2500_ATTR_SWEEP_MARKER_ENABLED, sweepMarkerEnabled), 2, "Sweep Marker Enabled");
    
    viCheckParm(hmf2500_SetAttributeViReal64(instrSession, "",
        HMF2500_ATTR_SWEEP_MARKER_FREQUENCY, sweepMarkerFrequency), 3, "Sweep Marker Frequency");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);   
    return error;
}

//-----------------------------------------------------------------------------
// Configure Sweep Marker Style
//-----------------------------------------------------------------------------
/// HIFN  Configures the sweep marker. 
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR markerStyle/Sets the marker style.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ConfigureSweepMarkerStyle (ViSession instrSession,
                                                     ViInt32 markerStyle)
{
	ViStatus	error = VI_SUCCESS;
		checkErr (Rs_LockSession (instrSession, VI_NULL));  

    viCheckParm(hmf2500_SetAttributeViInt32(instrSession, "",
        HMF2500_ATTR_SWEEP_MARKER_STYLE, markerStyle), 2, "Marker Style");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
	return error;
}

//-----------------------------------------------------------------------------
// Configure Burst 
//-----------------------------------------------------------------------------
/// HIFN  Configures the burst parameters. 
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR burstEnabled/Sets the burst state.
/// HIPAR burstMode/Sets the burst mode.
/// HIPAR burstNCycles/Sets the number of repetition cycles of the BURST mode.
/// HIPAR burstInternalPeriod/Sets the internal period time of the BURST mode.
/// HIPAR burstPhase/Sets the phase of the BURST mode in degree.  
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ConfigureBurst (ViSession instrSession,
                                          ViBoolean burstEnabled,
                                          ViInt32 burstMode,
                                          ViInt32 burstNCycles,
                                          ViReal64 burstInternalPeriod,
                                          ViReal64 burstPhase)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(hmf2500_SetAttributeViBoolean(instrSession, "",
        HMF2500_ATTR_BURST_ENABLED, burstEnabled), 2, "Burst Enabled");
    
    viCheckParm(hmf2500_SetAttributeViInt32(instrSession, "",
        HMF2500_ATTR_BURST_MODE, burstMode), 3, "Burst Mode");
    
    viCheckParm(hmf2500_SetAttributeViInt32(instrSession, "",
        HMF2500_ATTR_BURST_N_CYCLES, burstNCycles), 4, "Burst N Cycles");
    
    viCheckParm(hmf2500_SetAttributeViReal64(instrSession, "",
        HMF2500_ATTR_BURST_INTERNAL_PERIOD, burstInternalPeriod), 5, "Burst Internal Period");
    
    viCheckParm(hmf2500_SetAttributeViReal64(instrSession, "",
        HMF2500_ATTR_BURST_PHASE, burstPhase), 6, "Burst Phase");
     
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

//-----------------------------------------------------------------------------
// Configure Display Trace
//-----------------------------------------------------------------------------
/// HIFN  Sets the display trace parameters.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR traceEnabled/Acitvates or deactivates the trace function.
/// HIPAR traceIntensity/Sets the trace intensity value.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ConfigureDisplayTrace (ViSession instrSession,
                                                 ViBoolean traceEnabled,
                                                 ViReal64 traceIntensity)
{
	ViStatus	error = VI_SUCCESS;
	checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(hmf2500_SetAttributeViBoolean(instrSession, "",
        HMF2500_ATTR_DISPLAY_TRACE_ENABLE, traceEnabled), 2, "Trace Enabled");
	
	if (traceEnabled == VI_TRUE)
	{
    
    viCheckParm(hmf2500_SetAttributeViReal64(instrSession, "",
        HMF2500_ATTR_DISPLAY_TRACE_INTENSITY, traceIntensity), 3, "Trace Intensity");
    
	}
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
	return error;
}

//-----------------------------------------------------------------------------
// Configure Display
//-----------------------------------------------------------------------------
/// HIFN  Sets the display grid and brightness parameters.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR grid/Sets the grid value.
/// HIPAR LEDBrightness/Selects the LED brightness.
/// HIPAR backlight/Sets the value of the backlight.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ConfigureDisplay (ViSession instrSession,
                                            ViReal64 grid,
											ViInt32 LEDBrightness,
											ViReal64 backlight)
{
	ViStatus	error = VI_SUCCESS;
	checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(hmf2500_SetAttributeViReal64(instrSession, "",
        HMF2500_ATTR_DISPLAY_GRID, grid), 2, "Grid");
    
    viCheckParm(hmf2500_SetAttributeViInt32(instrSession, "",
        HMF2500_ATTR_DISPLAY_LED_BRIGHTNESS, LEDBrightness), 3, "LED Brightness");
	
	viCheckParm(hmf2500_SetAttributeViReal64(instrSession, "",
        HMF2500_ATTR_DISPLAY_BACKLIGHT, backlight), 4, "Backlight");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
	return error;
}

//-----------------------------------------------------------------------------
// Configure Arb Function
//-----------------------------------------------------------------------------
/// HIFN  Selects one of the built-in arbitrary waveforms or  one of user-defined waveforms and load it into the internal memory.  
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR arbFunction/Selects one of the built-in arbitrary waveforms or one of user-defined waveforms and load it into the internal memory.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ConfigureArbFunction (ViSession instrSession,
                                                ViInt32 arbFunction)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckParm(hmf2500_SetAttributeViInt32(instrSession, "",
            HMF2500_ATTR_ARB_FUNCTION, arbFunction), 2, "Arb Function");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

//-----------------------------------------------------------------------------
// Configure Arb Data
//-----------------------------------------------------------------------------
/// HIFN  Define an arbitrary waveform.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR arraySize/Sets the size of the array for the arbitrary data.
/// HIPAR arbData/Permet to send the string of the waveform data. 
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ConfigureArbData (ViSession instrSession,
                                            ViInt32 arraySize, ViChar arbData[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar*     pbuffer = NULL; 
    ViChar      *p2buf;
	ViUInt16 intf = 0;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    p2buf = pbuffer = malloc (arraySize + 64);

    p2buf += sprintf (pbuffer, "DATA #%ld%ld",
                               (ViInt32)(floor(log10(arraySize)) + 1),
                               arraySize);

    memcpy (p2buf, arbData, arraySize); 
    p2buf[arraySize] = '\n';
    
    if (intf ==4)
	{
        viCheckErr( viSetAttribute (instrSession, VI_ATTR_ASRL_END_OUT, VI_ASRL_END_NONE));
	}
    else
	{
        viCheckErr( viSetAttribute (instrSession, VI_ATTR_SEND_END_EN, VI_FALSE));
	}
   	error = viWrite (instrSession, (ViPBuf)pbuffer, (ViUInt32)(p2buf - pbuffer) + arraySize +1, VI_NULL);
    if (intf ==4)
	{
        viCheckErr( viSetAttribute (instrSession, VI_ATTR_ASRL_END_OUT,
									VI_ASRL_END_TERMCHAR));
	}
    else
	{
        viCheckErr( viSetAttribute (instrSession, VI_ATTR_SEND_END_EN, VI_TRUE));
	}

Error:
    if (pbuffer)
        free (pbuffer);
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

//-----------------------------------------------------------------------------
// Configure Arb Data Modulation
//-----------------------------------------------------------------------------
/// HIFN  Define an arbitrary waveform for modulation.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR arraySize/Sets the size of the array for the arbitrary data.
/// HIPAR arbDataModulation/Permet to send the string of the waveform data. 
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ConfigureArbDataModulation (ViSession instrSession,
                                                      ViInt32 arraySize,
                                                      ViChar arbDataModulation[])
{
	ViStatus	error = VI_SUCCESS;
    ViChar*     pbuffer = NULL; 
    ViChar      *p2buf;
	ViUInt16 intf = 0;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    p2buf = pbuffer = malloc (arraySize + 64);

    p2buf += sprintf (pbuffer, "DATA:MOD #%ld%ld",
                               (ViInt32)(floor(log10(arraySize)) + 1),
                               arraySize);

    memcpy (p2buf, arbDataModulation, arraySize);
    p2buf[arraySize] = '\n';
	
    if (intf ==4)
	{
        viCheckErr( viSetAttribute (instrSession, VI_ATTR_ASRL_END_OUT, VI_ASRL_END_NONE));
	}
    else
	{
        viCheckErr( viSetAttribute (instrSession, VI_ATTR_SEND_END_EN, VI_FALSE));
	}
   	error = viWrite (instrSession, (ViPBuf)pbuffer, (ViUInt32)(p2buf - pbuffer) + arraySize +1, VI_NULL);
    if (intf ==4)
	{
        viCheckErr( viSetAttribute (instrSession, VI_ATTR_ASRL_END_OUT,
									VI_ASRL_END_TERMCHAR));
	}
    else
	{
        viCheckErr( viSetAttribute (instrSession, VI_ATTR_SEND_END_EN, VI_TRUE));
	}
Error:
    if (pbuffer)
        free (pbuffer);
    Rs_UnlockSession(instrSession, VI_NULL);
	return error;
}

//-----------------------------------------------------------------------------
// Save Arb Data
//-----------------------------------------------------------------------------
/// HIFN  Allows to save the data of the arbitrary waveform.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_SaveArbData (ViSession instrSession)
{
	ViStatus	error = VI_SUCCESS;
	 checkErr (Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckParm(hmf2500_SetAttributeViString(instrSession, "",
            HMF2500_ATTR_DATA_SAVE, VI_NULL), 1, VI_NULL);    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
	return error;
}

//-----------------------------------------------------------------------------
// Set Attribute ViInt32, ViReal64, ViString, ViBoolean, ViSession 
//-----------------------------------------------------------------------------
/// HIFN These functions enable the instrument driver user to set
/// HIFN attribute values directly.  There are typesafe versions for
/// HIFN ViInt32, ViReal64, ViString, ViBoolean, and ViSession datatypes. 
//-----------------------------------------------------------------------------
/* ViInt32 */
ViStatus _VI_FUNC hmf2500_SetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViReal64 */
ViStatus _VI_FUNC hmf2500_SetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 value)
{
     return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViString */
ViStatus _VI_FUNC hmf2500_SetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViString value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/* ViBoolean */
ViStatus _VI_FUNC hmf2500_SetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViSession */
ViStatus _VI_FUNC hmf2500_SetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Get Attribute ViInt32, ViReal64, ViString, ViBoolean, ViSession                                                
//-----------------------------------------------------------------------------
/// HIFN These functions enable the instrument driver user to check
/// HIFN attribute values directly.  These functions check the value you
/// HIFN specify even if you set the rspr_ATTR_RANGE_CHECK
/// HIFN attribute to VI_FALSE.  There are typesafe versions for ViInt32,
/// HIFN ViReal64, ViString, ViBoolean, and ViSession datatypes.
//-----------------------------------------------------------------------------
/* ViInt32 */
ViStatus _VI_FUNC hmf2500_GetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViInt32), (ViInt32 *)value);
}

/* ViReal64 */
ViStatus _VI_FUNC hmf2500_GetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViReal64), (ViReal64 *)value);
}

/* ViString */
ViStatus _VI_FUNC hmf2500_GetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViInt32 bufSize,
                                                 ViChar value[])
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            bufSize, value);
}

/* ViBoolean */
ViStatus _VI_FUNC hmf2500_GetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViBoolean), (ViBoolean *)value);
}

/* ViSession */
ViStatus _VI_FUNC hmf2500_GetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViSession), (ViSession *)value);
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Get Attribute Repeated Capability Id(s)                                                
//-----------------------------------------------------------------------------
/// HIFN This function returns the repeated capability id(s) that belongs 
/// HIFN to the attribute you specify. More then one id is listed in comma 
/// HIFN separated string.
/// HIPAR instrSession/The ViSession handle that you obtain from the rspr_init or rspr_InitWithOptions function. 
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Repeated Capability Id(s) parameter.
/// HIPAR repeatedCapabilityIds/The buffer in which the function returns the repeated capability id(s) belonging to the attribute
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_GetAttributeRepeatedCapabilityIds (ViSession instrSession,
                                                            ViUInt32 attributeID,
                                                            ViInt32 bufferSize,
                                                            ViChar _VI_FAR repeatedCapabilityIds[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapNameId[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapNameId (instrSession, attributeID, repCapNameId));

    strncpy (repeatedCapabilityIds, repCapNameId, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIds[(bufferSize > RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE : bufferSize] = '\0';

Error:
    return error;
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Get Attribute Repeated Capability Id Name(s)                                                
//-----------------------------------------------------------------------------
/// HIFN This function returns the repeated capability name(s) that 
/// HIFN belongs to the attribute and single repeated capability id you may 
/// HIFN specify. More then one name is listed in comma separated string.
/// HIPAR instrSession/The ViSession handle that you obtain from the rspr_init or rspr_InitWithOptions function. 
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR repeatedCapabilityId/Defines the Id of the repeated capability on which to query repeated capability id name(s).
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Repeated Capability Id Name(s) parameter.
/// HIPAR repeatedCapabilityIdNames/The buffer in which the function returns the repeated capability id name(s) belonging to the attribute.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_GetAttributeRepeatedCapabilityIdNames (ViSession instrSession,
                                                                ViUInt32 attributeID,
                                                                ViString repeatedCapabilityId,
                                                                ViInt32 bufferSize,
                                                                ViChar _VI_FAR repeatedCapabilityIdNames[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapName[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 4, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapName (instrSession,
                                           attributeID,
                                           repeatedCapabilityId,
                                           repCapName));

    strncpy (repeatedCapabilityIdNames, repCapName, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIdNames[(bufferSize > RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE : bufferSize] = '\0';

Error:
    return error;
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Check Attribute ViInt32, ViReal64, ViString, ViBoolean, ViSession                                                
//-----------------------------------------------------------------------------
/// HIFN These functions enable the instrument driver user to check
/// HIFN attribute values directly.  These functions check the value you
/// HIFN specify even if you set the rspr_ATTR_RANGE_CHECK
/// HIFN attribute to VI_FALSE.  There are typesafe versions for ViInt32,
/// HIFN ViReal64, ViString, ViBoolean, and ViSession datatypes.
//-----------------------------------------------------------------------------

/* ViInt32 */
ViStatus _VI_FUNC hmf2500_CheckAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViInt32 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViReal64 */
ViStatus _VI_FUNC hmf2500_CheckAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViReal64 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViString */
ViStatus _VI_FUNC hmf2500_CheckAttributeViString (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViString value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/* ViBoolean */
ViStatus _VI_FUNC hmf2500_CheckAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViBoolean value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViSession */
ViStatus _VI_FUNC hmf2500_CheckAttributeViSession (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViSession value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

//-----------------------------------------------------------------------------
// Configure System Sound
//-----------------------------------------------------------------------------
/// HIFN  Enables or desables the system sound.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmf2500_init or hmf2500_InitWithOptions function. 
/// HIPAR errorBeepEnabled/Activates or desactivates the error beeper.
/// HIPAR controlBeepEnabled/Activates or desactivates the control beeper. 
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ConfigureSystemSound (ViSession instrSession,
                                                ViBoolean errorBeepEnabled,
                                                ViBoolean controlBeepEnabled)
{
	ViStatus	error = VI_SUCCESS;
	checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(hmf2500_SetAttributeViBoolean(instrSession, "",
        HMF2500_ATTR_SYSTEM_SOUND_ERROR_BEEPER, errorBeepEnabled), 2, "Error Beep Enabled");
	
	viCheckParm(hmf2500_SetAttributeViBoolean(instrSession, "",
        HMF2500_ATTR_SYSTEM_SOUND_CONTROL_BEEPER, controlBeepEnabled), 3, "Control Beep Enabled");
	
Error:
	
	Rs_UnlockSession(instrSession, VI_NULL);
	
	return error;
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Reset                                                
//-----------------------------------------------------------------------------
/// HIFN Restarts the device (power off/on) and resets the instrument 
/// HIFN state to its default settings.
/// HIPAR instrSession/The ViSession handle that you obtain from the rspr_init or rspr_InitWithOptions function. 
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_reset (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
          viCheckErr( viWrite (instrSession,(ViBuf)"*RST\n",5,NULL));
          Rs_Delay(10.0);
         
        }
    
    checkErr( hmf2500_DefaultInstrSetup (instrSession));
    checkErr( hmf2500_CheckStatus (instrSession));

Error:

    (void) Rs_UnlockSession (instrSession, VI_NULL);

    return error;
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Self-Test                                                
//-----------------------------------------------------------------------------
/// HIFN This function runs the instrument's self test routine and returns 
/// HIFN the test result(s). 
/// HIPAR instrSession/The ViSession handle that you obtain from the rspr_init or rspr_InitWithOptions function. 
/// HIPAR testResult/Contains the value returned from the instrument self test.  Zero means success.
/// HIPAR testMessage/Contains the string returned from the self test.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_self_test (ViSession instrSession, ViInt16 *testResult,
                                      ViChar testMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE];
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (testResult == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Test Result");
    if (testMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Test Message");

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
        //viCheckErr( viWrite(instrSession,(ViBuf)"*TST?\n",6,NULL));
        //viCheckErr( viWrite(instrSession,(ViBuf)"*OPC\n",5,NULL));
       
        viCheckErr( viRead(instrSession,(ViPBuf)buffer,(ViUInt32)strlen(buffer),NULL));
        checkErr(sscanf(buffer,"%hd", testResult));
        if (*testResult == 0)
            strcpy (testMessage, "Self-Test Passed.");
        else
            strcpy (testMessage, "Self-Test Failed.");
        }
    else
        {
        /* Simulate Self Test */
        *testResult = 0;
        strcpy (testMessage, "No error.");
        }

    checkErr( hmf2500_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Revision Query                                                
//-----------------------------------------------------------------------------
/// HIFN This function returns the revision numbers of the instrument 
/// HIFN driver and instrument firmware.
/// HIPAR instrSession/The ViSession handle that you obtain from the rspr_init or rspr_InitWithOptions function. 
/// HIPAR driverRev/Returns the instrument driver software revision numbers in the form of a string.
/// HIPAR instrRev/Returns the instrument firmware revision numbers in the form of a string.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_revision_query (ViSession instrSession, ViChar driverRev[],
                                           ViChar instrRev[])
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (driverRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Driver Revision");
    if (instrRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Instrument Revision");

    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_SPECIFIC_DRIVER_REVISION,
                                        0, 256, driverRev));
    checkErr( Rs_GetAttribute (instrSession, "", RS_ATTR_INSTRUMENT_FIRMWARE_REVISION,
                                        0, 256, instrRev));
    checkErr( hmf2500_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Error-Query                                                
//-----------------------------------------------------------------------------
/// HIFN This function reads an error code and a message from the 
/// HIFN instrument's error queue.
/// HIPAR instrSession/The ViSession handle that you obtain from the rspr_init or rspr_InitWithOptions function. 
/// HIPAR errCode/Returns the error code read from the instrument's error queue.
/// HIPAR errMessage/Returns the error message string read from the instrument's error message queue.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_error_query (ViSession instrSession, ViInt32 *errCode,
                                        ViChar errMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[HMF2500_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (errCode == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error Code");
    if (errMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
        viCheckErr (hmf2500_GetAttributeViString( instrSession, "", HMF2500_ATTR_SYSTEM_ERROR, HMF2500_IO_BUFFER_SIZE, buffer));
        if  (sscanf (buffer, "%ld,\"%[^\r\n\"]", errCode, errMessage) != 1)
            viCheckErr(RS_ERROR_UNEXPECTED_RESPONSE);

        }
    else
        {
            /* Simulate Error Query */
        *errCode = 0;
        strcpy (errMessage, "No error.");
        }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Error List                                                
//-----------------------------------------------------------------------------
/// HIFN This function reads all system messages and returns a list of 
/// HIFN comma separated strings. Each string corresponds to an entry in 
/// HIFN the table SYSTEM MESSAGES.
/// HIPAR instrSession/The ViSession handle that you obtain from the rspr_init or rspr_InitWithOptions function. 
/// HIPAR bufferSize/Sets the size of Errors string.
/// HIPAR errors/Returns the error message string read from the instrument's error message queue.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ErrorList (ViSession instrSession,
                                      ViInt32 bufferSize, 
                                      ViChar errors[])
{
    ViStatus    error   = VI_SUCCESS;
	bufferSize;
	errors;
    instrSession;

    error= RS_ERROR_FUNCTION_NOT_SUPPORTED;
    
    return error;
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// System Info                                               
//-----------------------------------------------------------------------------
/// HIFN  Reads the system informations of the instrument. 
/// HIPAR instrSession/The ViSession handle that you obtain from the rspr_init or rspr_InitWithOptions function. 
/// HIPAR serialNumber/Returns the instrument serial number in the form of a string.
/// HIPAR softwareVersion/Returns the instrument software version numbers in the form of a string.
/// HIPAR hardwareNumber/Returns the instrument hardware number in the form of a string.
/// HIPAR instrumentName/Returns the instrument name.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_SystemInfo (ViSession instrSession,
                                      ViChar serialNumber[],
                                      ViChar softwareVersion[],
                                      ViChar hardwareNumber[],
                                      ViChar instrumentName[])
{
	ViStatus	error = VI_SUCCESS;
	checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (serialNumber == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error Code");
    if (softwareVersion == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Code");
	if (hardwareNumber == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Null address for Error Code");
    if (instrumentName == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 5, "Null address for Error Code");
	

    checkErr( Rs_GetAttribute (instrSession, VI_NULL, HMF2500_ATTR_SYSTEM_SERIAL_NUMBER,
                                        0, 256, serialNumber));
    checkErr( Rs_GetAttribute (instrSession, "", HMF2500_ATTR_SYSTEM_SOFTWARE_VERSION,
                                        0, 256, softwareVersion));
	checkErr( Rs_GetAttribute (instrSession, VI_NULL, HMF2500_ATTR_SYSTEM_HARDWARE_NUMBER,
                                        0, 256, hardwareNumber));
    checkErr( Rs_GetAttribute (instrSession, "", HMF2500_ATTR_INSTRUMENT_NAME,
                                        0, 256, instrumentName));
	
    checkErr( hmf2500_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
	return error;
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Query Hardcopy Data                                              
//-----------------------------------------------------------------------------
/// HIFN  Reads the system informations of the instrument. 
/// HIPAR instrSession/The ViSession handle that you obtain from the rspr_init or rspr_InitWithOptions function. 
/// HIPAR data/Returns the hardcopy data.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_QueryHardcopyData (ViSession instrSession,
                                             ViUInt32 timeout, ViInt32 arraySize,
                                             ViChar data[],
                                             ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViUInt32    count = 0;
    ViUInt32    ret_count = 0;
    ViInt32    length = 0;
    ViChar      buffer[HMF2500_IO_BUFFER_SIZE] = "";
    ViChar      *dummy = NULL;
    ViBoolean   old_param = VI_FALSE;
    ViInt32     old_timeout=-1;
	ViUInt16    intf = 0;

    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
	if (data == NULL)
		viCheckParm (RS_ERROR_NULL_POINTER, 4, "Data");
    
    (void) viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &old_timeout);  
    (void) viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intf);
    (void) viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, timeout);
	viCheckErr( viGetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, &old_param)); 
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));
    
    if (intf ==4)
	{
        viCheckErr( viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE));
	}
    else
	{
        viCheckErr( viSetAttribute (instrSession, VI_ATTR_SUPPRESS_END_EN, VI_TRUE));
	}
	
    
    viCheckErr (viPrintf(instrSession, "HCOPy:DATA?\n"));
    //Read header       
    viCheckErr (viRead(instrSession, (ViPBuf)buffer, 2,  &ret_count));
    if (sscanf (buffer, "#%1ld", &count) != 1)
        checkErr(RS_ERROR_UNEXPECTED_RESPONSE); 

    viCheckErr (viRead (instrSession, (ViPBuf)buffer, (ViUInt32) count, &ret_count));
    if (sscanf (buffer, "%lu", &length)  != 1)
        checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
    
    checkAlloc (dummy = (ViChar *) calloc (length, sizeof(ViChar)));
    viCheckErr (viRead (instrSession, (ViBuf) dummy, length, &count));
	viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, old_param);
    if (intf ==4)
	{
        viCheckErr( viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN,
									VI_ASRL_END_TERMCHAR));
	}
    else
	{
        viCheckErr( viSetAttribute (instrSession, VI_ATTR_SUPPRESS_END_EN, VI_FALSE));
	}
	count = (length > arraySize) ? arraySize : length;
	memset (data, 0, arraySize);
	memcpy (data, dummy, count);
	if (returnedValues)
		*returnedValues = length;
	checkErr( hmf2500_CheckStatus (instrSession));
    
Error:
    if (old_timeout > 0) 
        (void) viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, old_timeout);
    if (dummy) free (dummy);
    Rs_UnlockSession(instrSession, VI_NULL);
	return error;
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Configure Hardcopy Format                                              
//-----------------------------------------------------------------------------
/// HIFN  Sets the hardcopy format.
/// HIPAR instrSession/The ViSession handle that you obtain from the rspr_init or rspr_InitWithOptions function. 
/// HIPAR hardcopyFormat/Selects the hardcopy format.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ConfigureHardcopyFormat (ViSession instrSession,
                                                   ViInt32 hardcopyFormat)
{
	ViStatus	error = VI_SUCCESS;
	checkErr (Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckParm(hmf2500_SetAttributeViInt32(instrSession, "",
            HMF2500_ATTR_HARDCOPY_FORMAT, hardcopyFormat), 2, "Hardcopy Format");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
	return error;
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Query Hardcopy Size                                           
//-----------------------------------------------------------------------------
/// HIFN  Sets the hardcopy format.
/// HIPAR instrSession/The ViSession handle that you obtain from the rspr_init or rspr_InitWithOptions function. 
/// HIPAR x/returns the hardcopy size dimensions x.
/// HIPAR y/returns the hardcopy size dimensions y. 
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_QueryHardcopySize (ViSession instrSession,
                                             ViInt32 *x, ViInt32 *y)
{
	ViStatus	error = VI_SUCCESS;
	checkErr (Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckParm(hmf2500_GetAttributeViInt32(instrSession, "",
            HMF2500_ATTR_QUERY_HARDCOPY_SIZE_X, x), 2, "X");
	
	viCheckParm(hmf2500_GetAttributeViInt32(instrSession, "",
            HMF2500_ATTR_QUERY_HARDCOPY_SIZE_Y, y), 3, "Y");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
	return error;
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Error Message                                                
//-----------------------------------------------------------------------------
/// HIFN This function converts a status code returned by an instrument 
/// HIFN driver function into a user-readable string. 
/// HIPAR instrSession/The ViSession handle that you obtain from the rspr_init or rspr_InitWithOptions function. 
/// HIPAR errorCode/Pass the Status parameter that is returned from any of the instrument driver functions.
/// HIPAR errorMessage/Returns the user-readable message string that corresponds to the status code you specify.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_error_message (ViSession instrSession, ViStatus errorCode,
                                          ViChar errorMessage[])
{
    ViStatus    error   = VI_SUCCESS;

    static      RsStringValueTable errorTable =
        {
        /*=CHANGE:================================================================*
            Insert instrument driver specific error codes here.  Example:

            {hmf2500_ERROR_TOO_MANY_SAMPLES,   "Sample Count cannot exceed 512."},
         *=============================================================END=CHANGE=*/
            HMF2500_ERROR_CODES_AND_MSGS,
            {VI_NULL, VI_NULL}
        };

    (void) Rs_LockSession(instrSession, VI_NULL);

        /* all VISA and RS error codes are handled as well as codes in the table */
    if (errorMessage == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    checkErr( Rs_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, errorTable));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

         
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Get Error                                                
//-----------------------------------------------------------------------------
/// HIFN This function retrieves and then clears the error information for 
/// HIFN the session or the current execution thread. One exception exists: 
/// HIFN If the BufferSize parameter is 0, the function does not clear the 
/// HIFN error information. By passing 0 for the buffer size, the caller 
/// HIFN can ascertain the buffer size required to get the entire error 
/// HIFN description string and then call the function again with a 
/// HIFN sufficiently large buffer.
/// HIFN 
/// HIFN If the user specifies a valid session for the InstrumentHandle 
/// HIFN parameter, Get Error retrieves and then clears the error 
/// HIFN information for the session. If the user passes VI_NULL for the 
/// HIFN InstrumentHandle parameter, this function retrieves and then 
/// HIFN clears the error information for the current execution thread.  If 
/// HIFN the InstrumentHandle parameter is an invalid session, the function 
/// HIFN does nothing and returns an error. Normally, the error information 
/// HIFN describes the first error that occurred since the user last called 
/// HIFN rspr_GetError or rspr_ClearError.
/// HIPAR instrSession/The ViSession handle that you obtain from the rspr_init or rspr_InitWithOptions function. 
/// HIPAR errorCode/Returns the error code for the session or execution thread.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Description parameter.
/// HIPAR description/Returns the error description for the session or execution thread.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_GetError (ViSession instrSession,
                                     ViStatus *errorCode,
                                     ViInt32 bufferSize,
                                     ViChar description[])
{
    ViStatus    error       = VI_SUCCESS;
    ViStatus    primary     = VI_SUCCESS,
                secondary   = VI_SUCCESS;
    ViChar      elaboration[256]    = "";
    ViChar      errorMessage[1024]  = "";
    ViChar     *appendPoint = errorMessage; 
    ViInt32     actualSize  = 0;
    ViBoolean   locked      = VI_FALSE;

    /* Lock Session */
    checkErr( Rs_LockSession(instrSession, &locked));
    
   /*  Test for nulls and acquire error data*/ 
    if (bufferSize != 0)
       {
        if (errorCode == VI_NULL)
            {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error");
            }
        checkErr( Rs_GetErrorInfo (instrSession, &primary, &secondary, elaboration));
        }

    else
        {
        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_ERROR_ELABORATION, 0, 256, elaboration));
        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_SECONDARY_ERROR, 0, (ViInt32) sizeof (secondary), &secondary));
        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_PRIMARY_ERROR, 0, (ViInt32) sizeof (primary), &primary));
        }
        
    if (description == VI_NULL)
        {
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Null address for Description");
        } 
        
    /* Format data */
    if (primary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( hmf2500_error_message (instrSession, primary, msg));
        appendPoint += sprintf(appendPoint, "Primary Error: (Hex 0x%08X) %s\n", (unsigned int) primary, msg);
        }

    if (secondary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( hmf2500_error_message (instrSession, secondary, msg));
        appendPoint += sprintf(appendPoint, "Secondary Error: (Hex 0x%08X) %s\n", (unsigned int) secondary, msg);
        } 

    if (strlen (elaboration) > 0)
        {
        sprintf(appendPoint, "Elaboration: %s", elaboration);
        }

    actualSize = (ViInt32) (strlen(errorMessage) + 1);

    /* Prepare return values */
    if (bufferSize == 0)
        {
        error = (ViStatus) actualSize;
        }
    else
        {
        if (bufferSize > 0)
            {
            if (actualSize > bufferSize)
                {
                error = actualSize;
                actualSize = bufferSize;
                }
            }
        memcpy(description, errorMessage, (size_t)(actualSize-1));
        description[actualSize-1] = '\0';
        }

    if (errorCode)
        {
        *errorCode = primary;
        }
      
Error:
    /* Unlock Session */
    (void) Rs_UnlockSession (instrSession, &locked);
    return error;
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Clear Error                                                
//-----------------------------------------------------------------------------
/// HIFN This function clears the error code and error description for the 
/// HIFN session. If the user specifies a valid session for the 
/// HIFN InstrumentHandle parameter, this function clears the error 
/// HIFN information for the session. If the user passes VI_NULL for the Vi 
/// HIFN parameter, this function clears the error information for the 
/// HIFN current execution thread. If the Vi parameter is an invalid 
/// HIFN session, the function does nothing and returns an error.
/// HIFN The function clears the error code by setting it to VI_SUCCESS.  
/// HIFN If the error description string is non-NULL, the function 
/// HIFN de-allocates the error description string and sets the address to 
/// HIFN VI_NULL.
/// HIFN 
/// HIFN Maintaining the error information separately for each thread is 
/// HIFN useful if the user does not have a session handle to pass to the 
/// HIFN rspr_GetError function, which occurs when a call to rspr_init or 
/// HIFN rspr_InitWithOptions fails.
/// HIPAR instrSession/The ViSession handle that you obtain from the rspr_init or rspr_InitWithOptions function. 
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ClearError (ViSession instrSession)
{
    return Rs_ClearErrorInfo (instrSession);
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Write Instrument Data                                                
//-----------------------------------------------------------------------------
/// HIFN This function writes a user-specified string to the instrument.
/// HIFN 
/// HIFN Note:
/// HIFN 
/// HIFN This function bypasses attribute state caching. Therefore, when 
/// HIFN you call this function, the cached values for all attributes will 
/// HIFN be invalidated.
/// HIFN 
//-----------------------------------------------------------------------------

ViStatus _VI_FUNC hmf2500_WriteInstrData (ViSession instrSession, ViString writeBuffer)
{
    return Rs_WriteInstrData (instrSession, writeBuffer);
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Read Instrument Data                                                
//-----------------------------------------------------------------------------
/// HIFN This function reads data from the instrument.
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_ReadInstrData (ViSession instrSession, ViInt32 numBytes,
                                          ViChar rdBuf[], ViUInt32 *bytesRead)
{
    return Rs_ReadInstrData (instrSession, numBytes, rdBuf, bytesRead);
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Query ViBoolean                                                
//-----------------------------------------------------------------------------
/// HIFN This function queries the ViBoolean value.
/// HIFN 
/// HIFN 
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_QueryViBoolean (ViSession instrSession,
                                           ViString command, 
                                           ViBoolean *value)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer [10] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    
    viCheckErr (viQueryf (instrSession, "%s\n", "%[^\r\n]", command, buffer));
    if (strstr (buffer, "OFF") != NULL)
        *value = VI_FALSE;
    else
        *value = VI_TRUE;
            
    checkErr( hmf2500_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Query ViInt32                                                
//-----------------------------------------------------------------------------
/// HIFN This function queries the ViInt32 value.
/// HIFN 
/// HIFN 
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_QueryViInt32 (ViSession instrSession,
                                         ViString command, 
                                         ViInt32 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%ld", command, value));
    checkErr( hmf2500_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Query ViReal64                                                
//-----------------------------------------------------------------------------
/// HIFN This function queries the ViReal64 value.
/// HIFN 
/// HIFN 
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_QueryViReal64 (ViSession instrSession,
                                          ViString command, 
                                          ViReal64 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%le", command, value));
    checkErr( hmf2500_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Query ViString                                                
//-----------------------------------------------------------------------------
/// HIFN This function queries the ViString value.
/// HIFN 
/// HIFN 
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_QueryViString (ViSession instrSession,
                                          ViString command, 
                                          ViInt32 bufferSize,
                                          ViChar _VI_FAR value[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *pbuffer = NULL;
    ViUInt32    bytesRead;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE];
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
   
    checkErr(sprintf(buffer,"%s\n", command));
    viCheckErr (viWrite (instrSession, (ViBuf)buffer,(ViUInt32)strlen(buffer),NULL));
    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &pbuffer, &bytesRead));  
    if (bytesRead>=(ViUInt32)bufferSize)
    {
        strncpy(value, pbuffer, (size_t) (bufferSize-1));
        value[bufferSize-1]='\0';
        error = (ViStatus) (bytesRead+1);
    } 
    else
    {
        strncpy(value, pbuffer, (size_t) bytesRead); 
        value[bytesRead]='\0';   
    }
    checkErr( hmf2500_CheckStatus (instrSession)); 
Error:
    if (pbuffer) free (pbuffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Close                                                
//-----------------------------------------------------------------------------
/// HIFN This function performs the following operations:
/// HIFN 
/// HIFN - Closes the instrument I/O session.
/// HIFN 
/// HIFN - Destroys the instrument driver session and all of its 
/// HIFN attributes.
/// HIFN 
/// HIFN - Deallocates any memory resources the driver uses.
/// HIFN 
/// HIFN Notes:
/// HIFN 
/// HIFN (1) You must unlock the session before calling rspr_close.
/// HIFN 
/// HIFN (2) After calling rspr_close, you cannot use the instrument 
/// HIFN driver again until you call rspr_init or rspr_InitWithOptions.
/// HIFN 
/// HIFN 
//-----------------------------------------------------------------------------
ViStatus _VI_FUNC hmf2500_close (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
 
    viCheckErr(viWrite(instrSession,(ViBuf)"SYST:LOC\n",9,NULL));
    checkErr( hmf2500_RsClose (instrSession));

Error:

    (void) Rs_UnlockSession (instrSession, VI_NULL);
    (void) Rs_Dispose (instrSession);

    return error;
}

/*****************************************************************************
 * Function: hmf2500_RsClose
 * Purpose:  This function performs all of the drivers clean-up operations
 *           except for closing the RS session.  This function is called by
 *           hmf2500_close or by an RS class driver.
 *****************************************************************************/
ViStatus _VI_FUNC hmf2500_RsClose (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    /* Do not lock here. The caller manages the lock. */

    if(instrSession != 0)
        {
        ViSession   rmSession = VI_NULL;

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSession), &rmSession));
        
        viCheckErr(viWrite(instrSession,(ViBuf)"SYST:LOC\n",9,NULL));
        
        (void) viClose (instrSession);
        (void) viClose (rmSession);
        }
Error:    
    return error;
}

/*****************************************************************************
 *------------------------ Global Session Callbacks -------------------------*
 *****************************************************************************/
/*****************************************************************************
 * Function: Rs_CheckStatusCallback
 * Purpose:  This function queries the instrument to determine if it has
 *           encountered an error. If the instrument has encountered an
 *           error, this function returns the RS_ERROR_INSTRUMENT_SPECIFIC
 *           error code. This function is called by the
 *           Rs_CheckStatus utility function. The
 *           Rs_SetAttribute and Rs_GetAttribute functions invoke this
 *           function when the optionFlags parameter includes the
 *           RS_VAL_DIRECT_USER_CALL flag.
 *
 *           The user can disable calls to this function by setting the
 *           hmf2500_QUERY_INSTRUMENT_STATUS attribute to VI_FALSE.  The driver can
 *           disable the check status callback for a particular attribute by
 *           setting the RS_VAL_DONT_CHECK_STATUS flag.
 *
 *****************************************************************************/
ViStatus hmf2500_CheckStatusCallback (ViSession instrSession)
{
    ViUInt32    ret_cnt;
    ViStatus    error   = VI_SUCCESS;
    ViInt16     esr     = 0;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE];
    ViUInt16    interface = VI_INTF_GPIB;
    
    viCheckErr(viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &interface));
    if (Rs_QueryInstrStatus (instrSession) && !Rs_Simulating (instrSession))
        {
            if (interface == VI_INTF_TCPIP) {   
                Rs_Delay (0.5);
            } else {
                Rs_Delay (0.05);
            }
        }
    //viCheckErr (viWrite (instrSession, (ViBuf) "*STB?\n", 6, NULL));
    viCheckErr (viWrite (instrSession, (ViBuf) "*ESR?\n", 6, NULL));
    if (Rs_QueryInstrStatus (instrSession) && !Rs_Simulating (instrSession))
        {
            Rs_Delay (0.1);
        }
    viCheckErr (viRead (instrSession, (ViPBuf) buffer, RS_MAX_MESSAGE_BUF_SIZE, &ret_cnt));
    if(error==1073676293) error=0;
    buffer[ret_cnt] = '\0';
    //------------------ fix for slow instrument response  -----------------------
    if(strcmp(buffer,"\n")==0){
        Rs_Delay (1);
        viCheckErr (viRead (instrSession, (ViPBuf) buffer, RS_MAX_MESSAGE_BUF_SIZE, &ret_cnt));
        buffer[ret_cnt] = '\0';
    }
    //------------------- end fix for slow instrument response  ------------------
    esr = (ViInt16) atol (buffer);

    if ((esr & IEEE_488_2_ERROR_MASK) != 0)
    {
        viCheckErr (RS_ERROR_INSTR_SPECIFIC);
    }

Error:
    if(error==1073676293) error=0;
    return error;
}


 /*****************************************************************************
 * Function: hmf2500_CheckTimedStatusCallback
 * Purpose:  This function overrides standard calback funtion.
 *****************************************************************************/
ViStatus hmf2500_CheckTimedStatusCallback (ViSession instrSession)
 {
    ViUInt32    ret_cnt;
    ViStatus    error   = VI_SUCCESS;
    ViInt16     esr     = 0;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE];
    ViUInt16    interface = VI_INTF_GPIB;
    
 
    viCheckErr(viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &interface));
    if (Rs_QueryInstrStatus (instrSession) && !Rs_Simulating (instrSession))
        {
            if (interface == VI_INTF_TCPIP) {   
                Rs_Delay (4);
            } else {
                Rs_Delay (0.05);
            }
        }
    //viCheckErr (viWrite (instrSession, (ViBuf) "*STB?\n", 6, NULL));
    if (Rs_QueryInstrStatus (instrSession) && !Rs_Simulating (instrSession))
        {
            Rs_Delay (0.1);
        }
    viCheckErr (viRead (instrSession, (ViPBuf) buffer, RS_MAX_MESSAGE_BUF_SIZE, &ret_cnt));
    if(error==1073676293) error=0;
    buffer[ret_cnt] = '\0';
    //------------------ fix for slow instrument response  -----------------------
    if(strcmp(buffer,"\n")==0){
        Rs_Delay (1);
        viCheckErr (viRead (instrSession, (ViPBuf) buffer, RS_MAX_MESSAGE_BUF_SIZE, &ret_cnt));
        buffer[ret_cnt] = '\0';
    }
    //------------------- end fix for slow instrument response  ------------------ 
    esr = (ViInt16) atol (buffer);

    if ((esr & IEEE_488_2_ERROR_MASK) != 0)
    {
        viCheckErr (RS_ERROR_INSTR_SPECIFIC);
    }

Error:
    if(error==1073676293) error=0;
    return error;
 }


/*****************************************************************************
 * Function: Rs_WaitForOPCCallback
 * Purpose:  This function waits until the instrument has finished processing
 *           all pending operations. This function is called by the
 *           Rs_WaitForOPC utility function. The RS engine invokes
 *           this function in the following two cases:
 *           - Before invoking the read callback for attributes for which the
 *             RS_VAL_WAIT_FOR_OPC_BEFORE_READS flag is set.
 *           - After invoking the write callback for attributes for which the
 *             RS_VAL_WAIT_FOR_OPC_AFTER_WRITES flag is set.
 *
 *****************************************************************************/
ViStatus hmf2500_WaitForOPCCallback (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;
    /*

       Change this function to wait for operation complete for your specific
       instrument. This example function is based on the IEEE 488.2 Common
       System Commands. This function assumes that the instrument does not
       generate service requests for other reasons. This example performs
       the following operations:

       - Enables the service request event.
       - Sends a command that forces the instrument to generate a service
         request.
       - Waits for the service request.
       - Cleans up after the service request.

    */
    
#if 1
    ViUInt16     response    = 0;
#else
    ViInt32     response    = 0;
#endif
    ViInt32     counter     = 0;
    ViUInt16    interface   = VI_INTF_GPIB;
    ViInt32     opcTimeout = 0;
    
    //viCheckErr(viWrite(instrSession,(ViBuf)"*OPC\n",5,NULL));
    
    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_OPC_TIMEOUT, 0,
        (ViInt32) sizeof (opcTimeout), &opcTimeout));

    viCheckErr( viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &interface));

    /* Interface specific operations */
    switch (interface)
        {
        case VI_INTF_ASRL: /* Serial interface */

            /* Not supported */

        break;
        default: /* Default case */

            /* Check STB for errors. */
            do
            {
#if 1
                error = viReadSTB (instrSession,  &response);
#else
               
                //viWrite(instrSession,"*STB?\n",6,NULL);
                viRead(instrSession,buffer,RS_MAX_MESSAGE_BUF_SIZE,NULL);
                sscanf(buffer,"%d", &response);
            
#endif

                Rs_Delay (0.001);
                counter++;

                if (counter >= opcTimeout)
                    return VI_ERROR_TMO;

                /* Wait while STB is non zero value. Only Error/Event Queue
                   and ESB bits are taken into consideration. */
            
            } while ((response & 0x24) == 0);
          
            return (((response & 0x4) != 0) && ((response & 0x20) == 0)) ? RS_ERROR_INSTRUMENT_STATUS : error;
        }

Error:

    return error;
}
